#!/usr/bin/python
# -*- coding: UTF-8 -*-


import math

class GPS_calc(object):
    """ Berechnungen, Formatierungen etc. """

    def __init__(self):
        # Konstanten
        #self.Erdradius = 6378388
        #self.lat_Dist  = (self.Erdradius * math.pi) / 180
        #print(self.lat_Dist)
        self.lat_Dist = 111323

        self.sat = [""] * 13    # 12 Satelliten + 1x Anzahl der Satelliten = 13

        # letzter Koordinatenwert mit dezimaler Minuten/Sekundendarstellung
        self.px       = 0
        self.py       = 0

        # Koordinaten des Referenzpunktes in dezimaler Minuten/Sekundendarstellung
        self.ref_x   = 0
        self.ref_y   = 0

        # MinMax-Werte auf Ausgangswert setzen
        self.clr_minmax()


    def check_refpunkt(self):
        """ Bei Programmstart ist noch kein Referenzpunkt definiert,
            wenn der Button "anzeigen" angeklickt wird, dann wird
            der Abstand zu den "Nullwerten" angezeigt - sieht doof aus.
            Daher wird bei anklicken von "anzeigen" geprüft, ob ein
            Referenzpunkt gewählt ist, wenn nicht, dann wird er gesetzt.
        """
        if self.ref_x == 0:
            self.set_refpunkt()


    def set_refpunkt(self):
        """ Den Referenzpunkt auf die letzte aktuelle Position setzen """
        self.ref_x = self.px
        self.ref_y = self.py


    def clr_minmax(self):
        """ Ausgangspunkt zur Bestimmung der Minimal/Maximalwerte:
			min = 360
			max = 0
        """
        self.max_y   = 0
        self.min_y   = 90
        self.max_x   = 0
        self.min_x   = 360


    def _make_date(self, t):
        """ Wandelt einen String ins Datumsformat um
            121010 -> 12.10.10
        """
        if len(t) >5:
            return "{}.{}.{}".format(t[:2], t[2:4], t[4:6])
        else:
            return "  .  .  "


    def _make_time(self, t):
        """ Wandelt einen String ins Zeitformat um
            125931 -> 12:59:31
        """
        if len(t) >5:
            return "{}:{}:{}".format(t[:2], t[2:4], t[4:6])
        else:
            return "  :  :  "


    def _minute_2_dez(self, e):
        """ Rechnet die eingelesenen Werte im Format "GGGMM.DDD" in Dezimalgrad um
            -> die Minuten müssen auf Dezimalwert umgerechnet werden
            (mm.dddd * 10/6) /100
        """
        if len(e) == 0:
            return -1

        # Longitude hat 3 stellige Gradanzeige, Latitude nur 2 stellige
        p = e.find('.')

        if p == -1:
            return -1

        elif p == 4:
            grad = float(e[:2])
            m = float(e[2:]) * 10/6

        elif p == 5:
            grad = float(e[:3])
            m = float(e[3:]) * 10/6

        return grad + m / 100


    def abstand(self, p1_x, p1_y, p2_x, p2_y):
        """Berechnet den Abstand zwischen zwei Punkten in einer Ebene

           die Punkte werden durch die geographische Länge/Breite
           jeweils mit dezimalen Nachkommastellen bestimmt.

           Liefert als Tuple zurück:
           - Abstand in x-Richtung (Lon) in Metern mit 1 Nachkommastellen
           - Abstand in y-Richtung (Lat)
           - Abstand absolut.
        """
        # der Abstand in Nord-Süd-Richtung in konstant
        y = p2_y - p1_y
        y = y * self.lat_Dist

		# der Abstand in Ost-West-Richtung hängt ab von der Breite
        # angle ist die mittlere Breite
        angle = abs((p1_y + p2_y) / 2)
        # dort beträgt der Abstand zwischen zwei Breitengraden:
        lon_Dist = math.cos(angle * math.pi / 180) * self.lat_Dist

        # hier den Abstand zwischen den Punkten in Ost-West-Grad berechnen
        # und mit dem dort wirksamen Abstand
        x = (p2_x - p1_x) * lon_Dist
        # nun noch den absoluten Abstand nach Pythagoras berechnen
        dist  = math.sqrt(x ** 2 + y ** 2)

        return (round(x,1), round(y,1), round(dist,1))


    def _calc_dist(self):
        """ Berechnet den Abstand zwischen den Min-Maxwerten """
        # Letzten Standort auf Extremwerte untersuchen
        self._check_min_max()

        x_delta, y_delta, abs_delta = \
                self.abstand(self.max_x, self.max_y, self.min_x, self.min_y)
        return(abs(x_delta), abs(y_delta), abs_delta)


    def _calc_ref_dist(self):
        """ Berechnet den Abstand zum Referenzpunkt """
        x_ref_delta, y_ref_delta, abs_ref_delta = \
                self.abstand(self.ref_x, self.ref_y, self.px, self.py )
        return(x_ref_delta, y_ref_delta, abs_ref_delta)


    def _check_min_max(self):
        """ Prüft, ob der letzte Punkt neuer Extrempunkte liefert """
        if self.py > self.max_y:
            self.max_y = self.py

        if self.px > self.max_x:
            self.max_x = self.px

        if self.py < self.min_y:
            self.min_y = self.py

        if self.px < self.min_x:
            self.min_x = self.px


# --- RMC ----------------------------------------------------------------------
# die Variablen für den Datensatz RMC löschen
    def rmc_clr(self):
        self.rmc_time    = ""
        self.rmc_date    = ""
        self.rmc_lat     = ""
        self.rmc_lon     = ""
        self.rmc_speed   = ""
        self.rmc_heading = ""


# --- den Datensatz RMC interpretieren und die Werte in Variablen ablegen
    def read_RMC(self, t):

        # Data set quality: 'A' = valid, 'V' = invalid
        if t[2] == 'A':

            #zeit = t[1]
            self.rmc_time = self._make_time(t[1])
            #date = t[9]
            # umformatieren im Minuten und Sekunden
            self.rmc_date = self._make_date(t[9])
            self.rmc_lon = "0{}° {}' {}".format(t[3][:2], t[3][2:], t[4])
            self.rmc_lat = "{}° {}' {}".format(t[5][:3], t[5][3:], t[6])

            if len(t[7]):
                self.rmc_speed = round((float(t[7]) * 1.852), 1)
            else:
                self.rmc_speed = 0

            # die aktuellen Koordinaten mit dezimalen Minuten speichern
            self.py = self._minute_2_dez(t[3])
            self.px = self._minute_2_dez(t[5])

            self.rmc_speed = "{} km/h".format(self.rmc_speed)
            self.rmc_heading = "{} °".format(t[8])

        else:
            self.rmc_clr()
            self.rmc_time = "Invalid"

        return ((self.rmc_time, self.rmc_date,  self.rmc_lat, self.rmc_lon,\
                    self.rmc_speed, self.rmc_heading))


# --- GSA ------------------------------------------------------------------------------------
#
# --------------------------------------------------------------------------------------------

    def read_GSA(self, t):
        """Satellitenwerte, PDOP, HDOP, VDOP"""
        self.gsa_sats1 = "{} {} {} {} {} {}".format(t[3], t[4], t[5], t[6], t[7], t[8])
        self.gsa_sats2 = "{} {} {} {} {} {}".format(t[9], t[10], t[11], t[12], t[13], t[14])
        self.gsa_pdop  = (t[15])
        self.gsa_hdop  = (t[16])
        self.gsa_vdop  = (t[17])

        return((self.gsa_sats1, self.gsa_sats2, self.gsa_pdop, self.gsa_hdop, self.gsa_vdop))

# --- GGA ------------------------------------------------------------------------------------
#
# --------------------------------------------------------------------------------------------

    def read_GGA(self, t):
        """Anzahl der Satelliten, Höhe geoid, Höhe x"""
        try:
            self.gga_sats    = t[7]
            self.gga_height1 = "{} m".format(t[9])
            self.gga_height2 = "{} m".format(t[11])
        except:
            print("Fehler in read_GGA")
            return(" ? ", " ? ", " ? ")

        return(( self.gga_sats, self.gga_height1, self.gga_height2))

# --- VTG ---------------------------------------------------------------------
#
# -----------------------------------------------------------------------------

    def read_VTG(self, t):
        """Geschwindigkeit, Kurs"""
        self.vtg_speed_kmh = "{} km/h".format(t[7])
        self.vtg_kurs_map  = "{} °".format(t[1])

        return((self.vtg_speed_kmh, self.vtg_kurs_map))

# --- GSV ---------------------------------------------------------------------
#
# -----------------------------------------------------------------------------

    def read_GSV(self, t):

        self.sat[0] = t[3] # Anzahl der sichtbaren Satelliten
        frames = t[1]
        frame = t[2]
        sats  = t[3]

        if frame == '1':
            j = 1
        elif frame == '2':
            j = 5
        elif frame == '3':
            j = 9

        i = 4
        while  i < 18:
            if i <= (len(t) - 4):
                if (len(t[i])) > 0:
                    self.sat[j] = "#{} el: {} az: {} db: {}".format(t[i], t[i+1], t[i+2], t[i+3])
            else:
                self.sat[j] = "-"
            i += 4
            j += 1

        if t[1] == t[2]:
            return((self.sat, int(frame)))
        else:
            return(None)


# --- GLL ---------------------------------------------------------------------
#
# -----------------------------------------------------------------------------
    def read_GLL(self, t):

        if t[6] == "A":
            self.gll_lat = "0{}° {}' {}".format(t[1][0:2], t[1][2:], t[2])
            self.gll_lon = "{}° {}' {}".format(t[3][0:3], t[3][3:], t[4])

            self.gll_time = self._make_time(t[5])

        else:
            self.gll_lat  = "invalid"
            self.gll_lon  = "invalid"
            self.gll_time = "invalid"

        return((self.gll_lon, self.gll_lat, self.gll_time))


if __name__ is "__main__":

    xr = "1000.0000"
    yr = "5200.0000"

    x = "1001.0000"
    y = "5201.0000"

    g =  GPS_calc()
    g.px = g._minute_2_dez(xr)
    g.py = g._minute_2_dez(yr)
    g.set_refpunkt()

    g.px = g._minute_2_dez(x)
    g.py = g._minute_2_dez(y)

    print(g.px, g.py, g.ref_x, g.ref_y)
    print(g._calc_ref_dist())
